@@ -101,7 +101,7 @@ mm_modem_location_get_capabilities (MMModemLocation *self)
 /*****************************************************************************/
 
 /**
- * mm_modem_location_get_capabilities:
+ * mm_modem_location_get_enabled:
  * @self: A #MMModemLocation.
  *
  * Gets a bitmask of the location capabilities which are enabled in this #MMModemLocation.
@@ -225,6 +225,172 @@ mm_modem_location_setup_sync (MMModemLocation *self,
 
 /*****************************************************************************/
 
+/**
+ * mm_modem_location_set_supl_server_finish:
+ * @self: A #MMModemLocation.
+ * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to mm_modem_location_set_supl_server().
+ * @error: Return location for error or %NULL.
+ *
+ * Finishes an operation started with mm_modem_location_set_supl_server().
+ *
+ * Returns: %TRUE if setting the SUPL server was successful, %FALSE if @error is set.
+ */
+gboolean
+mm_modem_location_set_supl_server_finish (MMModemLocation *self,
+                                          GAsyncResult *res,
+                                          GError **error)
+{
+    g_return_val_if_fail (MM_IS_MODEM_LOCATION (self), FALSE);
+
+    return mm_gdbus_modem_location_call_set_supl_server_finish (MM_GDBUS_MODEM_LOCATION (self), res, error);
+}
+
+/**
+ * mm_modem_location_set_supl_server:
+ * @self: A #MMModemLocation.
+ * @supl: The SUPL server address, given as IP:PORT or with a full URL.
+ * @cancellable: (allow-none): A #GCancellable or %NULL.
+ * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
+ * @user_data: User data to pass to @callback.
+ *
+ * Asynchronously configures the address of the SUPL server for A-GPS operation.
+ *
+ * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
+ * You can then call mm_modem_location_set_supl_server_finish() to get the result of the operation.
+ *
+ * See mm_modem_location_set_supl_server_sync() for the synchronous, blocking version of this method.
+ */
+void
+mm_modem_location_set_supl_server (MMModemLocation *self,
+                                   const gchar *supl,
+                                   GCancellable *cancellable,
+                                   GAsyncReadyCallback callback,
+                                   gpointer user_data)
+{
+    g_return_if_fail (MM_IS_MODEM_LOCATION (self));
+
+    mm_gdbus_modem_location_call_set_supl_server (MM_GDBUS_MODEM_LOCATION (self),
+                                                  supl,
+                                                  cancellable,
+                                                  callback,
+                                                  user_data);
+}
+
+/**
+ * mm_modem_location_set_supl_server_sync:
+ * @self: A #MMModemLocation.
+ * @supl: The SUPL server address, given as IP:PORT or with a full URL.
+ * @cancellable: (allow-none): A #GCancellable or %NULL.
+ * @error: Return location for error or %NULL.
+ *
+ * Synchronously configures the address of the SUPL server for A-GPS operation.
+ *
+ * The calling thread is blocked until a reply is received. See mm_modem_location_set_supl_server()
+ * for the asynchronous version of this method.
+ *
+ * Returns: %TRUE if setting the SUPL server was successful, %FALSE if @error is set.
+ */
+gboolean
+mm_modem_location_set_supl_server_sync (MMModemLocation *self,
+                                        const gchar *supl,
+                                        GCancellable *cancellable,
+                                        GError **error)
+{
+    g_return_val_if_fail (MM_IS_MODEM_LOCATION (self), FALSE);
+
+    return mm_gdbus_modem_location_call_set_supl_server_sync (MM_GDBUS_MODEM_LOCATION (self),
+                                                              supl,
+                                                              cancellable,
+                                                              error);
+}
+
+/*****************************************************************************/
+
+/**
+ * mm_modem_location_set_gps_refresh_rate_finish:
+ * @self: A #MMModemLocation.
+ * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to mm_modem_location_set_gps_refresh_rate().
+ * @error: Return location for error or %NULL.
+ *
+ * Finishes an operation started with mm_modem_location_set_gps_refresh_rate().
+ *
+ * Returns: %TRUE if setting the GPS refresh rate was successful, %FALSE if @error is set.
+ */
+gboolean
+mm_modem_location_set_gps_refresh_rate_finish (MMModemLocation *self,
+                                               GAsyncResult *res,
+                                               GError **error)
+{
+    g_return_val_if_fail (MM_IS_MODEM_LOCATION (self), FALSE);
+
+    return mm_gdbus_modem_location_call_set_gps_refresh_rate_finish (MM_GDBUS_MODEM_LOCATION (self), res, error);
+}
+
+/**
+ * mm_modem_location_set_gps_refresh_rate:
+ * @self: A #MMModemLocation.
+ * @rate: The GPS refresh rate, in seconds.
+ * @cancellable: (allow-none): A #GCancellable or %NULL.
+ * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
+ * @user_data: User data to pass to @callback.
+ *
+ * Asynchronously configures the GPS refresh rate.
+
+ * If a 0 rate is used, the GPS location updates will be immediately propagated to the interface.
+ *
+ * When the operation is finished, @callback will be invoked in the <link linkend="g-main-context-push-thread-default">thread-default main loop</link> of the thread you are calling this method from.
+ * You can then call mm_modem_location_set_gps_refresh_rate_finish() to get the result of the operation.
+ *
+ * See mm_modem_location_set_gps_refresh_rate_sync() for the synchronous, blocking version of this method.
+ */
+void
+mm_modem_location_set_gps_refresh_rate (MMModemLocation *self,
+                                        guint rate,
+                                        GCancellable *cancellable,
+                                        GAsyncReadyCallback callback,
+                                        gpointer user_data)
+{
+    g_return_if_fail (MM_IS_MODEM_LOCATION (self));
+
+    mm_gdbus_modem_location_call_set_gps_refresh_rate (MM_GDBUS_MODEM_LOCATION (self),
+                                                       rate,
+                                                       cancellable,
+                                                       callback,
+                                                       user_data);
+}
+
+/**
+ * mm_modem_location_set_gps_refresh_rate_sync:
+ * @self: A #MMModemLocation.
+ * @rate: The GPS refresh rate, in seconds.
+ * @cancellable: (allow-none): A #GCancellable or %NULL.
+ * @error: Return location for error or %NULL.
+ *
+ * Synchronously configures the GPS refresh rate.
+ *
+ * If a 0 rate is used, the GPS location updates will be immediately propagated to the interface.
+ *
+ * The calling thread is blocked until a reply is received. See mm_modem_location_set_gps_refresh_rate()
+ * for the asynchronous version of this method.
+ *
+ * Returns: %TRUE if setting the refresh rate was successful, %FALSE if @error is set.
+ */
+gboolean
+mm_modem_location_set_gps_refresh_rate_sync (MMModemLocation *self,
+                                             guint rate,
+                                             GCancellable *cancellable,
+                                             GError **error)
+{
+    g_return_val_if_fail (MM_IS_MODEM_LOCATION (self), FALSE);
+
+    return mm_gdbus_modem_location_call_set_gps_refresh_rate_sync (MM_GDBUS_MODEM_LOCATION (self),
+                                                                   rate,
+                                                                   cancellable,
+                                                                   error);
+}
+
+/*****************************************************************************/
+
 static gboolean
 build_locations (GVariant *dictionary,
                  MMLocation3gpp **location_3gpp,
@@ -389,7 +555,7 @@ mm_modem_location_get_full_sync (MMModemLocation *self,
  *
  * Finishes an operation started with mm_modem_location_get_3gpp().
  *
- * Returns: (transfer full) A #MMLocation3gpp, or #NULL if not available. The returned value should be freed with g_object_unref().
+ * Returns: (transfer full): A #MMLocation3gpp, or #NULL if not available. The returned value should be freed with g_object_unref().
  */
 MMLocation3gpp *
 mm_modem_location_get_3gpp_finish (MMModemLocation *self,
@@ -437,7 +603,7 @@ mm_modem_location_get_3gpp (MMModemLocation *self,
  * The calling thread is blocked until a reply is received. See mm_modem_location_get_3gpp()
  * for the asynchronous version of this method.
  *
- * Returns: (transfer full) A #MMLocation3gpp, or #NULL if not available. The returned value should be freed with g_object_unref().
+ * Returns: (transfer full): A #MMLocation3gpp, or #NULL if not available. The returned value should be freed with g_object_unref().
  */
 MMLocation3gpp *
 mm_modem_location_get_3gpp_sync (MMModemLocation *self,
@@ -461,7 +627,7 @@ mm_modem_location_get_3gpp_sync (MMModemLocation *self,
  *
  * Finishes an operation started with mm_modem_location_get_gps_nmea().
  *
- * Returns: (transfer full) A #MMLocationGpsNmea, or #NULL if not available. The returned value should be freed with g_object_unref().
+ * Returns: (transfer full): A #MMLocationGpsNmea, or #NULL if not available. The returned value should be freed with g_object_unref().
  */
 MMLocationGpsNmea *
 mm_modem_location_get_gps_nmea_finish (MMModemLocation *self,
@@ -509,7 +675,7 @@ mm_modem_location_get_gps_nmea (MMModemLocation *self,
  * The calling thread is blocked until a reply is received. See mm_modem_location_get_gps_nmea()
  * for the asynchronous version of this method.
  *
- * Returns: (transfer full) A #MMLocationGpsNmea, or #NULL if not available. The returned value should be freed with g_object_unref().
+ * Returns: (transfer full): A #MMLocationGpsNmea, or #NULL if not available. The returned value should be freed with g_object_unref().
  */
 MMLocationGpsNmea *
 mm_modem_location_get_gps_nmea_sync (MMModemLocation *self,
@@ -533,7 +699,7 @@ mm_modem_location_get_gps_nmea_sync (MMModemLocation *self,
  *
  * Finishes an operation started with mm_modem_location_get_gps_raw().
  *
- * Returns: (transfer full) A #MMLocationGpsRaw, or #NULL if not available. The returned value should be freed with g_object_unref().
+ * Returns: (transfer full): A #MMLocationGpsRaw, or #NULL if not available. The returned value should be freed with g_object_unref().
  */
 MMLocationGpsRaw *
 mm_modem_location_get_gps_raw_finish (MMModemLocation *self,
@@ -581,7 +747,7 @@ mm_modem_location_get_gps_raw (MMModemLocation *self,
  * The calling thread is blocked until a reply is received. See mm_modem_location_get_gps_raw()
  * for the asynchronous version of this method.
  *
- * Returns: (transfer full) A #MMLocationGpsRaw, or #NULL if not available. The returned value should be freed with g_object_unref().
+ * Returns: (transfer full): A #MMLocationGpsRaw, or #NULL if not available. The returned value should be freed with g_object_unref().
  */
 MMLocationGpsRaw *
 mm_modem_location_get_gps_raw_sync (MMModemLocation *self,
@@ -605,7 +771,7 @@ mm_modem_location_get_gps_raw_sync (MMModemLocation *self,
  *
  * Finishes an operation started with mm_modem_location_get_cdma_bs().
  *
- * Returns: (transfer full) A #MMLocationCdmaBs, or #NULL if not available. The returned value should be freed with g_object_unref().
+ * Returns: (transfer full): A #MMLocationCdmaBs, or #NULL if not available. The returned value should be freed with g_object_unref().
  */
 MMLocationCdmaBs *
 mm_modem_location_get_cdma_bs_finish (MMModemLocation *self,
@@ -653,7 +819,7 @@ mm_modem_location_get_cdma_bs (MMModemLocation *self,
  * The calling thread is blocked until a reply is received. See mm_modem_location_get_cdma_bs()
  * for the asynchronous version of this method.
  *
- * Returns: (transfer full) A #MMLocationCdmaBs, or #NULL if not available. The returned value should be freed with g_object_unref().
+ * Returns: (transfer full): A #MMLocationCdmaBs, or #NULL if not available. The returned value should be freed with g_object_unref().
  */
 MMLocationCdmaBs *
 mm_modem_location_get_cdma_bs_sync (MMModemLocation *self,
@@ -669,6 +835,65 @@ mm_modem_location_get_cdma_bs_sync (MMModemLocation *self,
 
 /*****************************************************************************/
 
+/**
+ * mm_modem_location_get_supl_server:
+ * @self: A #MMModemLocation.
+ *
+ * Gets the address of the SUPL server.
+ *
+ * <warning>The returned value is only valid until the property changes so
+ * it is only safe to use this function on the thread where
+ * @self was constructed. Use mm_modem_location_dup_supl_server() if on another
+ * thread.</warning>
+ *
+ * Returns: (transfer none): The SUPL server address, or %NULL if none available. Do not free the returned value, it belongs to @self.
+ */
+const gchar *
+mm_modem_location_get_supl_server (MMModemLocation *self)
+{
+    g_return_val_if_fail (MM_IS_MODEM_LOCATION (self), NULL);
+
+    RETURN_NON_EMPTY_CONSTANT_STRING (
+        mm_gdbus_modem_location_get_supl_server (MM_GDBUS_MODEM_LOCATION (self)));
+}
+
+/**
+ * mm_modem_location_dup_supl_server:
+ * @self: A #MMModemLocation.
+ *
+ * Gets the address of the SUPL server.
+ *
+ * Returns: (transfer full): The SUPL server address, or %NULL if none available. The returned value should be freed with g_free().
+ */
+gchar *
+mm_modem_location_dup_supl_server (MMModemLocation *self)
+{
+    g_return_val_if_fail (MM_IS_MODEM_LOCATION (self), NULL);
+
+    RETURN_NON_EMPTY_STRING (
+        mm_gdbus_modem_location_dup_supl_server (MM_GDBUS_MODEM_LOCATION (self)));
+}
+
+/*****************************************************************************/
+
+/**
+ * mm_modem_location_get_gps_refresh_rate:
+ * @self: A #MMModemLocation.
+ *
+ * Gets the GPS refresh rate, in seconds.
+ *
+ * Returns: The GPS refresh rate, or 0 if no fixed rate is used.
+ */
+guint
+mm_modem_location_get_gps_refresh_rate (MMModemLocation *self)
+{
+    g_return_val_if_fail (MM_IS_MODEM_LOCATION (self), 0);
+
+    return mm_gdbus_modem_location_get_gps_refresh_rate (MM_GDBUS_MODEM_LOCATION (self));
+}
+
+/*****************************************************************************/
+
 static void
 mm_modem_location_init (MMModemLocation *self)
 {
