@@ -32,6 +32,7 @@
 #include "mm-broadband-bearer-hso.h"
 #include "mm-log.h"
 #include "mm-modem-helpers.h"
+#include "mm-daemon-enums-types.h"
 
 G_DEFINE_TYPE (MMBroadbandBearerHso, mm_broadband_bearer_hso, MM_TYPE_BROADBAND_BEARER);
 
@@ -49,7 +50,7 @@ struct _MMBroadbandBearerHsoPrivate {
 typedef struct {
     MMBroadbandBearerHso *self;
     MMBaseModem *modem;
-    MMAtSerialPort *primary;
+    MMPortSerialAt *primary;
     guint cid;
     GSimpleAsyncResult *result;
 } GetIpConfig3gppContext;
@@ -99,7 +100,7 @@ ip_config_ready (MMBaseModem *modem,
     guint i;
     guint dns_i;
 
-    response = mm_base_modem_at_command_full_finish (MM_BASE_MODEM (modem), res, &error);
+    response = mm_base_modem_at_command_full_finish (modem, res, &error);
     if (error) {
         g_simple_async_result_take_error (ctx->result, error);
         get_ip_config_context_complete_and_free (ctx);
@@ -182,10 +183,11 @@ ip_config_ready (MMBaseModem *modem,
 static void
 get_ip_config_3gpp (MMBroadbandBearer *self,
                     MMBroadbandModem *modem,
-                    MMAtSerialPort *primary,
-                    MMAtSerialPort *secondary,
+                    MMPortSerialAt *primary,
+                    MMPortSerialAt *secondary,
                     MMPort *data,
                     guint cid,
+                    MMBearerIpFamily ip_family,
                     GAsyncReadyCallback callback,
                     gpointer user_data)
 {
@@ -222,7 +224,7 @@ get_ip_config_3gpp (MMBroadbandBearer *self,
 typedef struct {
     MMBroadbandBearerHso *self;
     MMBaseModem *modem;
-    MMAtSerialPort *primary;
+    MMPortSerialAt *primary;
     guint cid;
     GCancellable *cancellable;
     GSimpleAsyncResult *result;
@@ -314,44 +316,56 @@ connect_reset (Dial3gppContext *ctx)
     g_free (command);
 }
 
-void
-mm_broadband_bearer_hso_report_connection_status (MMBroadbandBearerHso *self,
-                                                  MMBroadbandBearerHsoConnectionStatus status)
+static void
+report_connection_status (MMBaseBearer *bearer,
+                          MMBearerConnectionStatus status)
 {
+    MMBroadbandBearerHso *self = MM_BROADBAND_BEARER_HSO (bearer);
     Dial3gppContext *ctx;
 
+    g_assert (status == MM_BEARER_CONNECTION_STATUS_CONNECTED ||
+              status == MM_BEARER_CONNECTION_STATUS_CONNECTION_FAILED ||
+              status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED);
+
     /* Recover context (if any) and remove both cancellation and timeout (if any)*/
     ctx = self->priv->connect_pending;
     self->priv->connect_pending = NULL;
 
+    /* Connection status reported but no connection attempt? */
+    if (!ctx) {
+        g_assert (self->priv->connect_pending_id == 0);
+
+        mm_dbg ("Received spontaneous _OWANCALL (%s)",
+                mm_bearer_connection_status_get_string (status));
+
+        if (status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED) {
+            /* If no connection attempt on-going, make sure we mark ourselves as
+             * disconnected */
+            MM_BASE_BEARER_CLASS (mm_broadband_bearer_hso_parent_class)->report_connection_status (
+                bearer,
+                status);
+        }
+        return;
+    }
+
     if (self->priv->connect_pending_id) {
         g_source_remove (self->priv->connect_pending_id);
         self->priv->connect_pending_id = 0;
     }
 
-    if (ctx && self->priv->connect_cancellable_id) {
+    if (self->priv->connect_cancellable_id) {
         g_cancellable_disconnect (ctx->cancellable,
                                   self->priv->connect_cancellable_id);
         self->priv->connect_cancellable_id = 0;
     }
 
-    if (ctx && self->priv->connect_port_closed_id) {
+    if (self->priv->connect_port_closed_id) {
         g_signal_handler_disconnect (ctx->primary, self->priv->connect_port_closed_id);
         self->priv->connect_port_closed_id = 0;
     }
 
-    switch (status) {
-    case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_UNKNOWN:
-        break;
-
-    case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTED:
-        if (!ctx)
-            /* We may get this if the timeout for the connection attempt is
-             * reached before the unsolicited response. We should probably
-             * keep the CID around to request explicit disconnection in this
-             * case. */
-            break;
-
+    /* Reporting connected */
+    if (status == MM_BEARER_CONNECTION_STATUS_CONNECTED) {
         /* If we wanted to get cancelled before, do it now */
         if (ctx->saved_error) {
             /* Keep error */
@@ -367,52 +381,23 @@ mm_broadband_bearer_hso_report_connection_status (MMBroadbandBearerHso *self,
                                                    (GDestroyNotify)g_object_unref);
         dial_3gpp_context_complete_and_free (ctx);
         return;
+    }
 
-    case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTION_FAILED:
-        if (!ctx)
-            break;
-
-        /* If we wanted to get cancelled before and now we couldn't connect,
-         * use the cancelled error and return */
-        if (ctx->saved_error) {
-            g_simple_async_result_take_error (ctx->result, ctx->saved_error);
-            ctx->saved_error = NULL;
-            dial_3gpp_context_complete_and_free (ctx);
-            return;
-        }
-
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Call setup failed");
+    /* If we wanted to get cancelled before and now we couldn't connect,
+     * use the cancelled error and return */
+    if (ctx->saved_error) {
+        g_simple_async_result_take_error (ctx->result, ctx->saved_error);
+        ctx->saved_error = NULL;
         dial_3gpp_context_complete_and_free (ctx);
         return;
-
-    case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_DISCONNECTED:
-        if (ctx) {
-            /* If we wanted to get cancelled before and now we couldn't connect,
-             * use the cancelled error and return */
-            if (ctx->saved_error) {
-                g_simple_async_result_take_error (ctx->result, ctx->saved_error);
-                ctx->saved_error = NULL;
-                dial_3gpp_context_complete_and_free (ctx);
-                return;
-            }
-
-            g_simple_async_result_set_error (ctx->result,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_FAILED,
-                                             "Call setup failed");
-            dial_3gpp_context_complete_and_free (ctx);
-            return;
-        }
-
-        /* Just ensure we mark ourselves as being disconnected... */
-        mm_bearer_report_disconnection (MM_BEARER (self));
-        return;
     }
 
-    g_warn_if_reached ();
+    /* Received CONNECTION_FAILED or DISCONNECTED during a connection attempt? */
+    g_simple_async_result_set_error (ctx->result,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_FAILED,
+                                     "Call setup failed");
+    dial_3gpp_context_complete_and_free (ctx);
 }
 
 static gboolean
@@ -453,7 +438,7 @@ connect_timed_out_cb (MMBroadbandBearerHso *self)
     /* It's probably pointless to try to reset this here, but anyway... */
     connect_reset (ctx);
 
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 static void
@@ -478,13 +463,12 @@ connect_cancelled_cb (GCancellable *cancellable,
 }
 
 static void
-forced_close_cb (MMSerialPort *port,
+forced_close_cb (MMPortSerial *port,
                  MMBroadbandBearerHso *self)
 {
     /* Just treat the forced close event as any other unsolicited message */
-    mm_broadband_bearer_hso_report_connection_status (
-        self,
-        MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTION_FAILED);
+    mm_base_bearer_report_connection_status (MM_BASE_BEARER (self),
+                                             MM_BEARER_CONNECTION_STATUS_CONNECTION_FAILED);
 }
 
 static void
@@ -585,12 +569,12 @@ authenticate_ready (MMBaseModem *modem,
 }
 
 const gchar *auth_commands[] = {
-	"$QCPDPP",
-	/* Icera-based devices (GI0322/Quicksilver, iCON 505) don't implement
-	 * $QCPDPP, but instead use _OPDPP with the same arguments.
-	 */
-	"_OPDPP",
-	NULL
+    "$QCPDPP",
+    /* Icera-based devices (GI0322/Quicksilver, iCON 505) don't implement
+     * $QCPDPP, but instead use _OPDPP with the same arguments.
+     */
+    "_OPDPP",
+    NULL
 };
 
 static void
@@ -610,14 +594,14 @@ authenticate (Dial3gppContext *ctx)
         return;
     }
 
-    user = mm_bearer_properties_get_user (mm_bearer_peek_config (MM_BEARER (ctx->self)));
-    password = mm_bearer_properties_get_password (mm_bearer_peek_config (MM_BEARER (ctx->self)));
-    allowed_auth = mm_bearer_properties_get_allowed_auth (mm_bearer_peek_config (MM_BEARER (ctx->self)));
+    user = mm_bearer_properties_get_user (mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self)));
+    password = mm_bearer_properties_get_password (mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self)));
+    allowed_auth = mm_bearer_properties_get_allowed_auth (mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self)));
 
     /* Both user and password are required; otherwise firmware returns an error */
     if (!user || !password || allowed_auth == MM_BEARER_ALLOWED_AUTH_NONE) {
         mm_dbg ("Not using authentication");
-		command = g_strdup_printf ("%s=%d,0",
+        command = g_strdup_printf ("%s=%d,0",
                                    auth_commands[ctx->auth_idx],
                                    ctx->cid);
     } else {
@@ -649,8 +633,8 @@ authenticate (Dial3gppContext *ctx)
             return;
         }
 
-        quoted_user = mm_at_serial_port_quote_string (user);
-        quoted_password = mm_at_serial_port_quote_string (password);
+        quoted_user = mm_port_serial_at_quote_string (user);
+        quoted_password = mm_port_serial_at_quote_string (password);
         command = g_strdup_printf ("%s=%d,%u,%s,%s",
                                    auth_commands[ctx->auth_idx],
                                    ctx->cid,
@@ -676,7 +660,7 @@ authenticate (Dial3gppContext *ctx)
 static void
 dial_3gpp (MMBroadbandBearer *self,
            MMBaseModem *modem,
-           MMAtSerialPort *primary,
+           MMPortSerialAt *primary,
            guint cid,
            GCancellable *cancellable,
            GAsyncReadyCallback callback,
@@ -722,7 +706,7 @@ dial_3gpp (MMBroadbandBearer *self,
 typedef struct {
     MMBroadbandBearerHso *self;
     MMBaseModem *modem;
-    MMAtSerialPort *primary;
+    MMPortSerialAt *primary;
     GSimpleAsyncResult *result;
 } DisconnectContext;
 
@@ -753,7 +737,7 @@ disconnect_owancall_ready (MMBaseModem *modem,
     GError *error = NULL;
 
     /* Ignore errors for now */
-    mm_base_modem_at_command_full_finish (MM_BASE_MODEM (modem), res, &error);
+    mm_base_modem_at_command_full_finish (modem, res, &error);
     if (error) {
         mm_dbg ("Disconnection failed (not fatal): %s", error->message);
         g_error_free (error);
@@ -766,8 +750,8 @@ disconnect_owancall_ready (MMBaseModem *modem,
 static void
 disconnect_3gpp (MMBroadbandBearer *self,
                  MMBroadbandModem *modem,
-                 MMAtSerialPort *primary,
-                 MMAtSerialPort *secondary,
+                 MMPortSerialAt *primary,
+                 MMPortSerialAt *secondary,
                  MMPort *data,
                  guint cid,
                  GAsyncReadyCallback callback,
@@ -803,7 +787,7 @@ disconnect_3gpp (MMBroadbandBearer *self,
 
 /*****************************************************************************/
 
-MMBearer *
+MMBaseBearer *
 mm_broadband_bearer_hso_new_finish (GAsyncResult *res,
                                     GError **error)
 {
@@ -818,9 +802,9 @@ mm_broadband_bearer_hso_new_finish (GAsyncResult *res,
         return NULL;
 
     /* Only export valid bearers */
-    mm_bearer_export (MM_BEARER (bearer));
+    mm_base_bearer_export (MM_BASE_BEARER (bearer));
 
-    return MM_BEARER (bearer);
+    return MM_BASE_BEARER (bearer);
 }
 
 void
@@ -836,8 +820,8 @@ mm_broadband_bearer_hso_new (MMBroadbandModemHso *modem,
         cancellable,
         callback,
         user_data,
-        MM_BEARER_MODEM, modem,
-        MM_BEARER_CONFIG, config,
+        MM_BASE_BEARER_MODEM, modem,
+        MM_BASE_BEARER_CONFIG, config,
         NULL);
 }
 
@@ -845,7 +829,7 @@ static void
 mm_broadband_bearer_hso_init (MMBroadbandBearerHso *self)
 {
     /* Initialize private data */
-    self->priv = G_TYPE_INSTANCE_GET_PRIVATE ((self),
+    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
                                               MM_TYPE_BROADBAND_BEARER_HSO,
                                               MMBroadbandBearerHsoPrivate);
 }
@@ -854,10 +838,12 @@ static void
 mm_broadband_bearer_hso_class_init (MMBroadbandBearerHsoClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    MMBaseBearerClass *base_bearer_class = MM_BASE_BEARER_CLASS (klass);
     MMBroadbandBearerClass *broadband_bearer_class = MM_BROADBAND_BEARER_CLASS (klass);
 
     g_type_class_add_private (object_class, sizeof (MMBroadbandBearerHsoPrivate));
 
+    base_bearer_class->report_connection_status = report_connection_status;
     broadband_bearer_class->dial_3gpp = dial_3gpp;
     broadband_bearer_class->dial_3gpp_finish = dial_3gpp_finish;
     broadband_bearer_class->get_ip_config_3gpp = get_ip_config_3gpp;
