@@ -150,7 +150,7 @@ cpms_timeout_cb (ModemAfterSimUnlockContext *ctx)
 {
     ctx->retries--;
     modem_after_sim_unlock_context_step (ctx);
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 static void
@@ -704,25 +704,9 @@ modem_3gpp_cleanup_unsolicited_events (MMIfaceModem3gpp *self,
 static void
 setup_ports (MMBroadbandModem *self)
 {
-    MMAtSerialPort *ports[2];
-    guint i;
-
     /* Call parent's setup ports first always */
     MM_BROADBAND_MODEM_CLASS (mm_broadband_modem_zte_parent_class)->setup_ports (self);
 
-    ports[0] = mm_base_modem_peek_port_primary (MM_BASE_MODEM (self));
-    ports[1] = mm_base_modem_peek_port_secondary (MM_BASE_MODEM (self));
-
-    /* Configure AT ports */
-    for (i = 0; i < 2; i++) {
-        if (!ports[i])
-            continue;
-
-        g_object_set (ports[i],
-                      MM_PORT_CARRIER_DETECT, FALSE,
-                      NULL);
-    }
-
     /* Now reset the unsolicited messages we'll handle when enabled */
     mm_common_zte_set_unsolicited_events_handlers (MM_BROADBAND_MODEM (self),
                                                    MM_BROADBAND_MODEM_ZTE (self)->priv->unsolicited_setup,
