@@ -53,6 +53,9 @@ const gchar *mm_strip_tag    (const gchar *str,
 
 gchar **mm_split_string_groups (const gchar *str);
 
+GArray *mm_parse_uint_list (const gchar  *str,
+                            GError      **error);
+
 guint mm_count_bits_set (gulong number);
 
 gchar *mm_create_device_identifier (guint vid,
@@ -103,6 +106,9 @@ GRegex    *mm_3gpp_cusd_regex_get (void);
 GRegex    *mm_3gpp_cmti_regex_get (void);
 GRegex    *mm_3gpp_cds_regex_get (void);
 
+/* AT+WS46=? response parser: returns array of MMModemMode values */
+GArray *mm_3gpp_parse_ws46_test_response (const gchar  *response,
+                                          GError      **error);
 
 /* AT+COPS=? (network scan) response parser */
 typedef struct {
@@ -221,6 +227,22 @@ gboolean mm_3gpp_parse_crsm_response (const gchar *reply,
                                       gchar **hex,
                                       GError **error);
 
+/* +CESQ response parser */
+gboolean mm_3gpp_parse_cesq_response (const gchar  *response,
+                                      guint        *out_rxlev,
+                                      guint        *out_ber,
+                                      guint        *out_rscp,
+                                      guint        *out_ecn0,
+                                      guint        *out_rsrq,
+                                      guint        *out_rsrp,
+                                      GError      **error);
+
+gboolean mm_3gpp_cesq_response_to_signal_info (const gchar  *response,
+                                               MMSignal    **out_gsm,
+                                               MMSignal    **out_umts,
+                                               MMSignal    **out_lte,
+                                               GError      **error);
+
 /* Additional 3GPP-specific helpers */
 
 MMModem3gppFacility mm_3gpp_acronym_to_facility (const gchar *str);
