@@ -25,12 +25,15 @@
 #include <libmm-glib.h>
 
 #include "mm-modem-helpers-qmi.h"
+#include "mm-iface-modem.h"
 #include "mm-iface-modem-messaging.h"
 #include "mm-sms-qmi.h"
 #include "mm-base-modem.h"
+#include "mm-sms-part-3gpp.h"
+#include "mm-sms-part-cdma.h"
 #include "mm-log.h"
 
-G_DEFINE_TYPE (MMSmsQmi, mm_sms_qmi, MM_TYPE_SMS);
+G_DEFINE_TYPE (MMSmsQmi, mm_sms_qmi, MM_TYPE_BASE_SMS)
 
 /*****************************************************************************/
 
@@ -43,10 +46,10 @@ ensure_qmi_client (MMSmsQmi *self,
 {
     MMBaseModem *modem = NULL;
     QmiClient *client;
-    MMQmiPort *port;
+    MMPortQmi *port;
 
     g_object_get (self,
-                  MM_SMS_MODEM, &modem,
+                  MM_BASE_SMS_MODEM, &modem,
                   NULL);
     g_assert (MM_IS_BASE_MODEM (modem));
 
@@ -63,9 +66,9 @@ ensure_qmi_client (MMSmsQmi *self,
         return FALSE;
     }
 
-    client = mm_qmi_port_peek_client (port,
+    client = mm_port_qmi_peek_client (port,
                                       service,
-                                      MM_QMI_PORT_FLAG_DEFAULT);
+                                      MM_PORT_QMI_FLAG_DEFAULT);
     if (!client) {
         g_simple_async_report_error_in_idle (G_OBJECT (self),
                                              callback,
@@ -81,11 +84,38 @@ ensure_qmi_client (MMSmsQmi *self,
     return TRUE;
 }
 
+/*****************************************************************************/
+
+static gboolean
+check_sms_type_support (MMSmsQmi *self,
+                        MMBaseModem *modem,
+                        MMSmsPart *first_part,
+                        GError **error)
+{
+    if (MM_SMS_PART_IS_3GPP (first_part) && !mm_iface_modem_is_3gpp (MM_IFACE_MODEM (modem))) {
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_UNSUPPORTED,
+                     "Non-3GPP modem doesn't support 3GPP SMS");
+        return FALSE;
+    }
+
+    if (MM_SMS_PART_IS_CDMA (first_part) && !mm_iface_modem_is_cdma (MM_IFACE_MODEM (modem))) {
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_UNSUPPORTED,
+                     "Non-CDMA modem doesn't support CDMA SMS");
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 /*****************************************************************************/
 /* Store the SMS */
 
 typedef struct {
-    MMSms *self;
+    MMBaseSms *self;
     MMBaseModem *modem;
     QmiClientWms *client;
     GSimpleAsyncResult *result;
@@ -96,7 +126,7 @@ typedef struct {
 static void
 sms_store_context_complete_and_free (SmsStoreContext *ctx)
 {
-    g_simple_async_result_complete (ctx->result);
+    g_simple_async_result_complete_in_idle (ctx->result);
     g_object_unref (ctx->result);
     g_object_unref (ctx->client);
     g_object_unref (ctx->modem);
@@ -105,7 +135,7 @@ sms_store_context_complete_and_free (SmsStoreContext *ctx)
 }
 
 static gboolean
-sms_store_finish (MMSms *self,
+sms_store_finish (MMBaseSms *self,
                   GAsyncResult *res,
                   GError **error)
 {
@@ -147,7 +177,7 @@ store_ready (QmiClientWms *client,
     qmi_message_wms_raw_write_output_unref (output);
 
     /* Set the index in the part we hold */
-    parts = mm_sms_get_parts (ctx->self);
+    parts = mm_base_sms_get_parts (ctx->self);
     mm_sms_part_set_index ((MMSmsPart *)parts->data, (guint)idx);
 
     /* Go on with next one */
@@ -159,7 +189,7 @@ static void
 sms_store_next_part (SmsStoreContext *ctx)
 {
     QmiMessageWmsRawWriteInput *input;
-    guint8 *pdu;
+    guint8 *pdu = NULL;
     guint pdulen = 0;
     guint msgstart = 0;
     GArray *array;
@@ -173,10 +203,22 @@ sms_store_next_part (SmsStoreContext *ctx)
     }
 
     /* Get PDU */
-    pdu = mm_sms_part_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
+    if (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data))
+        pdu = mm_sms_part_3gpp_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
+    else if (MM_SMS_PART_IS_CDMA ((MMSmsPart *)ctx->current->data))
+        pdu = mm_sms_part_cdma_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &error);
+
     if (!pdu) {
-        /* 'error' should already be set */
-        g_simple_async_result_take_error (ctx->result, error);
+        if (error)
+            g_simple_async_result_take_error (ctx->result, error);
+        else
+            g_simple_async_result_set_error (ctx->result,
+                                             MM_CORE_ERROR,
+                                             MM_CORE_ERROR_FAILED,
+                                             "Unknown or unsupported PDU type in SMS part: %s",
+                                             mm_sms_pdu_type_get_string (
+                                                 mm_sms_part_get_pdu_type (
+                                                     (MMSmsPart *)ctx->current->data)));
         sms_store_context_complete_and_free (ctx);
         return;
     }
@@ -192,7 +234,9 @@ sms_store_next_part (SmsStoreContext *ctx)
     qmi_message_wms_raw_write_input_set_raw_message_data (
         input,
         mm_sms_storage_to_qmi_storage_type (ctx->storage),
-        QMI_WMS_MESSAGE_FORMAT_GSM_WCDMA_POINT_TO_POINT,
+        (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data) ?
+         QMI_WMS_MESSAGE_FORMAT_GSM_WCDMA_POINT_TO_POINT :
+         QMI_WMS_MESSAGE_FORMAT_CDMA),
         array,
         NULL);
     qmi_client_wms_raw_write (ctx->client,
@@ -206,13 +250,14 @@ sms_store_next_part (SmsStoreContext *ctx)
 }
 
 static void
-sms_store (MMSms *self,
+sms_store (MMBaseSms *self,
            MMSmsStorage storage,
            GAsyncReadyCallback callback,
            gpointer user_data)
 {
     SmsStoreContext *ctx;
     QmiClient *client = NULL;
+    GError *error = NULL;
 
     /* Ensure WMS client */
     if (!ensure_qmi_client (MM_SMS_QMI (self),
@@ -230,10 +275,19 @@ sms_store (MMSms *self,
     ctx->client = g_object_ref (client);
     ctx->storage = storage;
     g_object_get (self,
-                  MM_SMS_MODEM, &ctx->modem,
+                  MM_BASE_SMS_MODEM, &ctx->modem,
                   NULL);
 
-    ctx->current = mm_sms_get_parts (self);
+    ctx->current = mm_base_sms_get_parts (self);
+
+    /* Check whether we support the given SMS type */
+    if (!check_sms_type_support (MM_SMS_QMI (self), ctx->modem, (MMSmsPart *)ctx->current->data, &error)) {
+        g_simple_async_result_take_error (ctx->result, error);
+        sms_store_context_complete_and_free (ctx);
+        return;
+    }
+
+    /* Go on */
     sms_store_next_part (ctx);
 }
 
@@ -241,7 +295,7 @@ sms_store (MMSms *self,
 /* Send the SMS */
 
 typedef struct {
-    MMSms *self;
+    MMBaseSms *self;
     MMBaseModem *modem;
     QmiClientWms *client;
     GSimpleAsyncResult *result;
@@ -261,7 +315,7 @@ sms_send_context_complete_and_free (SmsSendContext *ctx)
 }
 
 static gboolean
-sms_send_finish (MMSms *self,
+sms_send_finish (MMBaseSms *self,
                  GAsyncResult *res,
                  GError **error)
 {
@@ -325,16 +379,29 @@ static void
 sms_send_generic (SmsSendContext *ctx)
 {
     QmiMessageWmsRawSendInput *input;
-    guint8 *pdu;
+    guint8 *pdu = NULL;
     guint pdulen = 0;
     guint msgstart = 0;
     GArray *array;
     GError *error = NULL;
 
     /* Get PDU */
-    pdu = mm_sms_part_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
+    if (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data))
+        pdu = mm_sms_part_3gpp_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
+    else if (MM_SMS_PART_IS_CDMA ((MMSmsPart *)ctx->current->data))
+        pdu = mm_sms_part_cdma_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &error);
+
     if (!pdu) {
-        g_simple_async_result_take_error (ctx->result, error);
+        if (error)
+            g_simple_async_result_take_error (ctx->result, error);
+        else
+            g_simple_async_result_set_error (ctx->result,
+                                             MM_CORE_ERROR,
+                                             MM_CORE_ERROR_FAILED,
+                                             "Unknown or unsupported PDU type in SMS part: %s",
+                                             mm_sms_pdu_type_get_string (
+                                                 mm_sms_part_get_pdu_type (
+                                                     (MMSmsPart *)ctx->current->data)));
         sms_send_context_complete_and_free (ctx);
         return;
     }
@@ -346,10 +413,11 @@ sms_send_generic (SmsSendContext *ctx)
     g_free (pdu);
 
     input = qmi_message_wms_raw_send_input_new ();
-
     qmi_message_wms_raw_send_input_set_raw_message_data (
         input,
-        QMI_WMS_MESSAGE_FORMAT_GSM_WCDMA_POINT_TO_POINT,
+        (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data) ?
+         QMI_WMS_MESSAGE_FORMAT_GSM_WCDMA_POINT_TO_POINT :
+         QMI_WMS_MESSAGE_FORMAT_CDMA),
         array,
         NULL);
 
@@ -404,6 +472,8 @@ send_from_storage_ready (QmiClientWms *client,
         } else {
             QmiWmsGsmUmtsRpCause rp_cause;
             QmiWmsGsmUmtsTpCause tp_cause;
+            QmiWmsCdmaCauseCode cdma_cause_code;
+            QmiWmsCdmaErrorClass cdma_error_class;
 
             if (qmi_message_wms_send_from_memory_storage_output_get_gsm_wcdma_cause_info (
                     output,
@@ -417,6 +487,24 @@ send_from_storage_ready (QmiClientWms *client,
                          qmi_wms_gsm_umts_tp_cause_get_string (tp_cause));
             }
 
+            if (qmi_message_wms_send_from_memory_storage_output_get_cdma_cause_code (
+                    output,
+                    &cdma_cause_code,
+                    NULL)) {
+                mm_warn ("Couldn't send SMS; cause code (%u): '%s'",
+                         cdma_cause_code,
+                         qmi_wms_cdma_cause_code_get_string (cdma_cause_code));
+            }
+
+            if (qmi_message_wms_send_from_memory_storage_output_get_cdma_error_class (
+                    output,
+                    &cdma_error_class,
+                    NULL)) {
+                mm_warn ("Couldn't send SMS; error class (%u): '%s'",
+                         cdma_error_class,
+                         qmi_wms_cdma_error_class_get_string (cdma_error_class));
+            }
+
             g_prefix_error (&error, "Couldn't write SMS part: ");
             g_simple_async_result_take_error (ctx->result, error);
             sms_send_context_complete_and_free (ctx);
@@ -446,9 +534,11 @@ sms_send_from_storage (SmsSendContext *ctx)
 
     qmi_message_wms_send_from_memory_storage_input_set_information (
         input,
-        mm_sms_storage_to_qmi_storage_type (mm_sms_get_storage (ctx->self)),
+        mm_sms_storage_to_qmi_storage_type (mm_base_sms_get_storage (ctx->self)),
         mm_sms_part_get_index ((MMSmsPart *)ctx->current->data),
-        QMI_WMS_MESSAGE_MODE_GSM_WCDMA,
+        (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data) ?
+         QMI_WMS_MESSAGE_MODE_GSM_WCDMA :
+         QMI_WMS_MESSAGE_MODE_CDMA),
         NULL);
 
     qmi_client_wms_send_from_memory_storage (
@@ -479,12 +569,13 @@ sms_send_next_part (SmsSendContext *ctx)
 }
 
 static void
-sms_send (MMSms *self,
+sms_send (MMBaseSms *self,
           GAsyncReadyCallback callback,
           gpointer user_data)
 {
     SmsSendContext *ctx;
     QmiClient *client = NULL;
+    GError *error = NULL;
 
     /* Ensure WMS client */
     if (!ensure_qmi_client (MM_SMS_QMI (self),
@@ -501,20 +592,28 @@ sms_send (MMSms *self,
     ctx->self = g_object_ref (self);
     ctx->client = g_object_ref (client);
     g_object_get (self,
-                  MM_SMS_MODEM, &ctx->modem,
+                  MM_BASE_SMS_MODEM, &ctx->modem,
                   NULL);
 
     /* If the SMS is STORED, try to send from storage */
-    ctx->from_storage = (mm_sms_get_storage (self) != MM_SMS_STORAGE_UNKNOWN);
+    ctx->from_storage = (mm_base_sms_get_storage (self) != MM_SMS_STORAGE_UNKNOWN);
+
+    ctx->current = mm_base_sms_get_parts (self);
+
+    /* Check whether we support the given SMS type */
+    if (!check_sms_type_support (MM_SMS_QMI (self), ctx->modem, (MMSmsPart *)ctx->current->data, &error)) {
+        g_simple_async_result_take_error (ctx->result, error);
+        sms_send_context_complete_and_free (ctx);
+        return;
+    }
 
-    ctx->current = mm_sms_get_parts (self);;
     sms_send_next_part (ctx);
 }
 
 /*****************************************************************************/
 
 typedef struct {
-    MMSms *self;
+    MMBaseSms *self;
     MMBaseModem *modem;
     QmiClientWms *client;
     GSimpleAsyncResult *result;
@@ -534,7 +633,7 @@ sms_delete_parts_context_complete_and_free (SmsDeletePartsContext *ctx)
 }
 
 static gboolean
-sms_delete_finish (MMSms *self,
+sms_delete_finish (MMBaseSms *self,
                    GAsyncResult *res,
                    GError **error)
 {
@@ -604,7 +703,7 @@ delete_next_part (SmsDeletePartsContext *ctx)
     input = qmi_message_wms_delete_input_new ();
     qmi_message_wms_delete_input_set_memory_storage (
         input,
-        mm_sms_storage_to_qmi_storage_type (mm_sms_get_storage (ctx->self)),
+        mm_sms_storage_to_qmi_storage_type (mm_base_sms_get_storage (ctx->self)),
         NULL);
     qmi_message_wms_delete_input_set_memory_index (
         input,
@@ -612,7 +711,9 @@ delete_next_part (SmsDeletePartsContext *ctx)
         NULL);
     qmi_message_wms_delete_input_set_message_mode (
         input,
-        QMI_WMS_MESSAGE_MODE_GSM_WCDMA,
+        (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data) ?
+         QMI_WMS_MESSAGE_MODE_GSM_WCDMA:
+         QMI_WMS_MESSAGE_MODE_CDMA),
         NULL);
     qmi_client_wms_delete (ctx->client,
                            input,
@@ -624,7 +725,7 @@ delete_next_part (SmsDeletePartsContext *ctx)
 }
 
 static void
-sms_delete (MMSms *self,
+sms_delete (MMBaseSms *self,
             GAsyncReadyCallback callback,
             gpointer user_data)
 {
@@ -645,22 +746,22 @@ sms_delete (MMSms *self,
     ctx->self = g_object_ref (self);
     ctx->client = g_object_ref (client);
     g_object_get (self,
-                  MM_SMS_MODEM, &ctx->modem,
+                  MM_BASE_SMS_MODEM, &ctx->modem,
                   NULL);
 
     /* Go on deleting parts */
-    ctx->current = mm_sms_get_parts (self);
+    ctx->current = mm_base_sms_get_parts (self);
     delete_next_part (ctx);
 }
 
 /*****************************************************************************/
 
-MMSms *
+MMBaseSms *
 mm_sms_qmi_new (MMBaseModem *modem)
 {
-    return MM_SMS (g_object_new (MM_TYPE_SMS_QMI,
-                                 MM_SMS_MODEM, modem,
-                                 NULL));
+    return MM_BASE_SMS (g_object_new (MM_TYPE_SMS_QMI,
+                                      MM_BASE_SMS_MODEM, modem,
+                                      NULL));
 }
 
 static void
@@ -671,12 +772,12 @@ mm_sms_qmi_init (MMSmsQmi *self)
 static void
 mm_sms_qmi_class_init (MMSmsQmiClass *klass)
 {
-    MMSmsClass *sms_class = MM_SMS_CLASS (klass);
+    MMBaseSmsClass *base_sms_class = MM_BASE_SMS_CLASS (klass);
 
-    sms_class->store = sms_store;
-    sms_class->store_finish = sms_store_finish;
-    sms_class->send = sms_send;
-    sms_class->send_finish = sms_send_finish;
-    sms_class->delete = sms_delete;
-    sms_class->delete_finish = sms_delete_finish;
+    base_sms_class->store = sms_store;
+    base_sms_class->store_finish = sms_store_finish;
+    base_sms_class->send = sms_send;
+    base_sms_class->send_finish = sms_send_finish;
+    base_sms_class->delete = sms_delete;
+    base_sms_class->delete_finish = sms_delete_finish;
 }
